@@ -6,12 +6,12 @@
 **
 ************************************************************************
 **
-**   Copyright (c): 2010-2011 Axel Pauli
+**   Copyright (c): 2010-2012 Axel Pauli
 **
 **   This file is distributed under the terms of the General Public
 **   License. See the file COPYING for more information.
 **
-**   $Id: flarmaliaslist.cpp 4693 2011-10-20 14:47:50Z axel $
+**   $Id: flarmaliaslist.cpp 5013 2012-03-20 16:08:27Z axel $
 **
 ***********************************************************************/
 
@@ -40,7 +40,7 @@ FlarmAliasList::FlarmAliasList( QWidget *parent ) :
   topLayout->setSpacing(5);
 
   list = new QTableWidget( 0, 2, this );
-  list->setSelectionBehavior( QAbstractItemView::SelectRows );
+  list->setSelectionBehavior( QAbstractItemView::SelectItems );
 
   // hide vertical headers
   // QHeaderView *vHeader = list->verticalHeader();
@@ -62,13 +62,19 @@ FlarmAliasList::FlarmAliasList( QWidget *parent ) :
   connect( list, SIGNAL(cellChanged( int, int )),
            this, SLOT(slot_CellChanged( int, int )) );
 
+  connect( list, SIGNAL(cellClicked( int, int )),
+           this, SLOT(slot_CellClicked( int, int )) );
+
+  connect( list, SIGNAL(itemSelectionChanged()),
+           this, SLOT(slot_ItemSelectionChanged()) );
+
   topLayout->addWidget( list, 2 );
 
   QGroupBox* buttonBox = new QGroupBox( this );
 
   int size = 40;
 
-#ifdef MAEMO
+#if defined MAEMO || defined ANDROID
   size = 60;
 #endif
 
@@ -79,12 +85,13 @@ FlarmAliasList::FlarmAliasList( QWidget *parent ) :
   addButton->setMinimumSize(size, size);
   addButton->setMaximumSize(size, size);
 
-  QPushButton *deleteButton  = new QPushButton;
+  deleteButton  = new QPushButton;
   deleteButton->setIcon( QIcon( GeneralConfig::instance()->loadPixmap( "delete.png" ) ) );
   deleteButton->setIconSize(QSize(IconSize, IconSize));
   deleteButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::QSizePolicy::Preferred);
   deleteButton->setMinimumSize(size, size);
   deleteButton->setMaximumSize(size, size);
+  deleteButton->setEnabled(false);
 
   QPushButton *okButton = new QPushButton;
   okButton->setIcon(QIcon(GeneralConfig::instance()->loadPixmap("ok.png")));
@@ -137,7 +144,6 @@ FlarmAliasList::FlarmAliasList( QWidget *parent ) :
 
 FlarmAliasList::~FlarmAliasList()
 {
-  // qDebug() << "FlarmAliasList::~FlarmAliasList()";
 }
 
 void FlarmAliasList::showEvent( QShowEvent *event )
@@ -145,6 +151,15 @@ void FlarmAliasList::showEvent( QShowEvent *event )
   Q_UNUSED( event )
 
   list->resizeColumnToContents( 0 );
+
+  QString style = "QTableView QTableCornerButton::section { background: gray }";
+  list->setStyleSheet( style );
+
+  QHeaderView *vHeader = list->verticalHeader();
+
+  style = "QHeaderView::section { width: 2em }";
+
+  vHeader->setStyleSheet( style );
 }
 
 void FlarmAliasList::slot_AddRow( QString col0, QString col1 )
@@ -156,11 +171,11 @@ void FlarmAliasList::slot_AddRow( QString col0, QString col1 )
   QTableWidgetItem* item;
 
   item = new QTableWidgetItem( col0 );
-  item->setFlags( Qt::ItemIsSelectable| Qt::ItemIsEditable | Qt::ItemIsEnabled );
+  item->setFlags( Qt::ItemIsSelectable | Qt::ItemIsEnabled );
   list->setItem( row, 0, item );
 
   item = new QTableWidgetItem( col1 );
-  item->setFlags( Qt::ItemIsSelectable| Qt::ItemIsEditable | Qt::ItemIsEnabled );
+  item->setFlags( Qt::ItemIsSelectable | Qt::ItemIsEnabled );
   list->setItem( row, 1, item );
 
   if( ! col0.isEmpty() && col0 == FlarmDisplay::getSelectedObject() )
@@ -188,11 +203,23 @@ void FlarmAliasList::slot_DeleteRows()
       return;
     }
 
-  int answer= QMessageBox::question(this, tr("Delete"),
-                                   tr("Delete selected entries?"),
-                                   QMessageBox::No, QMessageBox::Yes);
+  QMessageBox mb( QMessageBox::Question,
+                  tr( "Delete?" ),
+                  tr( "Delete selected entries?" ),
+                  QMessageBox::Yes | QMessageBox::No,
+                  this );
 
-  if( answer == QMessageBox::No )
+  mb.setDefaultButton( QMessageBox::No );
+
+#ifdef ANDROID
+
+  mb.show();
+  QPoint pos = mapToGlobal(QPoint( width()/2 - mb.width()/2, height()/2 - mb.height()/2 ));
+  mb.move( pos );
+
+#endif
+
+  if( mb.exec() == QMessageBox::No )
     {
       return;
     }
@@ -237,9 +264,18 @@ void FlarmAliasList::slot_Ok()
             {
               if( list->item( i, j )->text().trimmed().isEmpty() )
                 {
-                  QMessageBox::warning(this,tr("Missing Entry"),
-                                       tr("Please fill out all fields!"),
-                                       QMessageBox::Ok );
+                  QMessageBox mb( QMessageBox::Warning,
+                                  tr( "Missing Entry" ),
+                                  tr( "Please fill out all fields!" ),
+                                  QMessageBox::Ok,
+                                  this );
+#ifdef ANDROID
+                  mb.show();
+                  QPoint pos = mapToGlobal(QPoint( width()/2 - mb.width()/2,
+                                                   height()/2 - mb.height()/2 ));
+                  mb.move( pos );
+#endif
+                  mb.exec();
                   return;
                 }
             }
@@ -315,6 +351,72 @@ void FlarmAliasList::slot_CellChanged( int row, int column )
     }
 }
 
+void FlarmAliasList::slot_CellClicked( int row, int column )
+{
+  QTableWidgetItem* item = list->item ( row, column );
+
+  if( item == static_cast<QTableWidgetItem *>(0) || row < 0 || column < 0 )
+    {
+      // Item can be a Null pointer, if a row has been removed.
+      return;
+    }
+
+  QString title, label;
+
+  if( column == 0 )
+    {
+      title = tr("Enter Flarm ID");
+      label = tr("Flarm ID:");
+    }
+  else
+    {
+      title = tr("Enter Alias");
+      label = tr("Alias (15):");
+    }
+
+  bool ok;
+  QString text = QInputDialog::getText( this,
+                                        title,
+                                        label,
+                                        QLineEdit::Normal,
+                                        item->text(),
+                                        &ok );
+  if( ok  )
+    {
+      item->setText( text );
+    }
+}
+
+void FlarmAliasList::slot_ItemSelectionChanged()
+{
+  bool enabled = false;
+
+  for( int i = 0; i < list->rowCount(); i++ )
+    {
+      QTableWidgetItem *item0 = list->item( i, 0 );
+      QTableWidgetItem *item1 = list->item( i, 1 );
+
+      if( item0 && item1 )
+        {
+          if( item0->isSelected() && item1->isSelected() )
+            {
+              enabled = true;
+              continue;
+            }
+
+          if( ! item0->isSelected() && ! item1->isSelected() )
+            {
+              continue;
+            }
+
+          enabled = false;
+          break;
+        }
+    }
+
+  deleteButton->setEnabled(enabled);
+}
+
 #warning "FLARM alias file 'cumulus-flarm.txt' is stored at User Data Directory"
 
 /** Loads the Flarm alias data from the related file into the alias hash. */
@@ -328,7 +430,7 @@ bool FlarmAliasList::loadAliasData()
   if ( ! f.open( QIODevice::ReadOnly ) )
     {
       // could not open file ...
-      qDebug() << "Cannot open file: " << f.fileName();
+      qWarning() << "Cannot open file: " << f.fileName();
       mutex.unlock();
       return false;
     }
