@@ -3,11 +3,11 @@
                              -------------------
     begin                : Sat Jul 20 2002
     copyright            : (C) 2002      by André Somers,
-                               2008-2010 by Axel Pauli
+                               2008-2012 by Axel Pauli
 
     email                : axel@kflog.org
 
-    $Id: gpsnmea.cpp 4600 2011-04-29 20:49:33Z axel $
+    $Id: gpsnmea.cpp 5039 2012-04-04 12:52:34Z axel $
 
  ***************************************************************************
  *                                                                         *
@@ -29,24 +29,29 @@
  * MAEMO4 for Maemo 4, OS2008, devices N8x0
  * MAEMO5 for Maemo 5, OS2009, devices N900
  *
+ * ANDROID for Android-Lighthouse Qt port (skipping all code referring to
+ * external devices, using system location service via JNI)
  */
 
-#include <stdio.h>
-#include <stdlib.h>
+#include <cstdio>
+#include <cstdlib>
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/time.h>
-#include <time.h>
+#include <ctime>
 #include <cmath>
 
 #include <QtCore>
 
+#include "generalconfig.h"
 #include "gpsnmea.h"
 #include "mapmatrix.h"
 #include "mapcalc.h"
 #include "mapview.h"
 
-#include "generalconfig.h"
+#ifdef ANDROID
+#include "androidevents.h"
+#endif
 
 #ifdef FLARM
 #include "flarm.h"
@@ -57,7 +62,6 @@
 #define FIX_TO 25000
 
 extern MapView   *_globalMapView;
-extern MapMatrix *_globalMapMatrix;
 
 // global GPS object pointer
 GpsNmea *GpsNmea::gps = static_cast<GpsNmea *> (0);
@@ -68,6 +72,9 @@ short GpsNmea::instances = 0;
 // Dictionary with known sentence keywords
 QHash<QString, short> GpsNmea::gpsHash;
 
+// Mutex for thread synchronization
+QMutex GpsNmea::mutex;
+
 GpsNmea::GpsNmea(QObject* parent) : QObject(parent)
 {
   if( instances > 0 )
@@ -78,41 +85,32 @@ GpsNmea::GpsNmea(QObject* parent) : QObject(parent)
 
   instances++;
 
+  _enableGpsDataProcessing = true;
+
   resetDataObjects();
 
-  // Load known GPS sentence identifier.
-  gpsHash.insert( "$GPRMC", 0);
-  gpsHash.insert( "$GPGLL", 1);
-  gpsHash.insert( "$GPGGA", 2);
-  gpsHash.insert( "$GPGSA", 3);
-  gpsHash.insert( "$GPGSV", 4);
-  gpsHash.insert( "$PGRMZ", 5);
-  gpsHash.insert( "$PCAID", 6);
-  gpsHash.insert( "!w",     7);
-  gpsHash.insert( "$PGCS",  8);
-  gpsHash.insert( "$LXWP0", 9);
-  gpsHash.insert( "$LXWP2", 10);
-  gpsHash.insert( "$GPDTM", 11);
-
-#ifdef FLARM
-  gpsHash.insert( "$PFLAA", 12);
-  gpsHash.insert( "$PFLAU", 13);
-#endif
-
-#ifdef MAEMO5
-  gpsHash.insert( "$MAEMO0", 14);
-  gpsHash.insert( "$MAEMO1", 15);
-#endif
-
-  gpsHash.squeeze();
+  // Load desired GPS sentence identifier into the hash filter.
+  getGpsMessageKeys(gpsHash);
 
   // GPS fix supervision, is started after the first fix was received
   timeOutFix = new QTimer(this);
   connect (timeOutFix, SIGNAL(timeout()), this, SLOT(_slotTimeoutFix()));
 
   // Load last used device
-  gpsDevice = GeneralConfig::instance()->getGpsDevice();
-  serial    = static_cast<GpsCon *> (0);
+  gpsDevice   = GeneralConfig::instance()->getGpsDevice();
+
+#ifndef ANDROID
+  serial = static_cast<GpsCon *> (0);
+#else
+  serial = static_cast<QObject *> (0);
+#endif
+
+  nmeaLogFile = static_cast<QFile *> (0);
+
+  if( GeneralConfig::instance()->getGpsNmeaLogState() == true )
+    {
+      slot_openNmeaLogFile();
+    }
 
   createGpsConnection();
 }
@@ -128,6 +126,49 @@ GpsNmea::~GpsNmea()
       delete serial;
       writeConfig();
     }
+
+  if( nmeaLogFile )
+    {
+      if( nmeaLogFile->isOpen() )
+        {
+          nmeaLogFile->close();
+        }
+
+      delete nmeaLogFile;
+    }
+}
+
+void GpsNmea::getGpsMessageKeys( QHash<QString, short>& gpsKeys)
+{
+  mutex.lock();
+  gpsKeys.clear();
+
+  // Load all desired GPS sentence identifier into the hash table
+  gpsKeys.insert( "$GPRMC", 0);
+  gpsKeys.insert( "$GPGLL", 1);
+  gpsKeys.insert( "$GPGGA", 2);
+  gpsKeys.insert( "$GPGSA", 3);
+  gpsKeys.insert( "$GPGSV", 4);
+  gpsKeys.insert( "$PGRMZ", 5);
+  gpsKeys.insert( "$PCAID", 6);
+  gpsKeys.insert( "!w",     7);
+  gpsKeys.insert( "$PGCS",  8);
+  gpsKeys.insert( "$LXWP0", 9);
+  gpsKeys.insert( "$LXWP2", 10);
+  gpsKeys.insert( "$GPDTM", 11);
+
+#ifdef FLARM
+  gpsKeys.insert( "$PFLAA", 12);
+  gpsKeys.insert( "$PFLAU", 13);
+#endif
+
+#ifdef MAEMO5
+  gpsKeys.insert( "$MAEMO0", 14);
+  gpsKeys.insert( "$MAEMO1", 15);
+#endif
+
+  gpsKeys.squeeze();
+  mutex.unlock();
 }
 
 /** Resets all data objects to their initial values. This is called
@@ -170,6 +211,9 @@ void GpsNmea::resetDataObjects()
   _lastUtc = QDateTime();
 
   _ignoreConnectionLost = false;
+  _gprmcSeen = false;
+
+  reportedUnknownKeys.clear();
 
 #ifdef FLARM
   pflaaIsReceiving = false;
@@ -180,12 +224,14 @@ void GpsNmea::resetDataObjects()
 
 void GpsNmea::createGpsConnection()
 {
+#ifndef ANDROID
+
   // qDebug("GpsNmea::createGpsConnection()");
   QObject *gpsObject = 0;
 
   // We create only a GpsCon instance. The GPS daemon process will be started
   // later. This is also valid hence Maemo5.
-  QString callPath = GeneralConfig::instance()->getInstallRoot() + "/bin";
+  QString callPath = GeneralConfig::instance()->getAppRoot() + "/bin";
 
   serial = new GpsCon(this, callPath.toAscii().data());
 
@@ -205,6 +251,7 @@ void GpsNmea::createGpsConnection()
   // The connection to the GPS receiver or daemon has been established
   connect (gpsObject, SIGNAL(gpsConnectionOn()),
            this, SLOT( _slotGpsConnectionOn()) );
+#endif
 }
 
 /**
@@ -214,30 +261,25 @@ void GpsNmea::createGpsConnection()
  */
 void GpsNmea::enableReceiving( bool enable )
 {
-  QSocketNotifier* notifier = static_cast<QSocketNotifier *> (0);
-
   // qDebug("GpsNmea::enableReceiving(%s)", enable ? "true" : "false");
+
+  _enableGpsDataProcessing = enable;
+
+#ifndef ANDROID
+
   if ( serial )
     {
+      QSocketNotifier* notifier = static_cast<QSocketNotifier *> (0);
+
       notifier = serial->getDaemonNotifier();
+
+      if( notifier )
+        {
+          notifier->setEnabled( enable );
+        }
     }
 
-  if( notifier )
-    {
-      notifier->setEnabled( enable );
-    }
-}
-
-/**
- * This method can be used to read data from the connected GPS. It is provided
- * to empty the receiver queue during long running other actions to avoid a
- * buffer overflow. At the moment only Maemo4 uses it because it is not clear
- * how long the MAEMO GPS daemon can buffer data.
- * For the Cumulus GPS daemon or hence Mameo5 with method is not necessary.
- * The Cumulus GPS daemon can queue 500 NMEA sentences internally.
- */
-void GpsNmea::readDataFromGps()
-{
+#endif
 }
 
 /**
@@ -245,10 +287,14 @@ void GpsNmea::readDataFromGps()
  */
 void GpsNmea::startGpsReceiver()
 {
+#ifndef ANDROID
+
   if ( serial )
     {
       serial->startClientProcess();
     }
+
+#endif
 }
 
 /**
@@ -275,6 +321,16 @@ void GpsNmea::startGpsReceiver()
 void GpsNmea::slot_sentence(const QString& sentenceIn)
 {
   // qDebug("GpsNmea::slot_sentence: %s", sentenceIn.toLatin1().data());
+  if( nmeaLogFile && nmeaLogFile->isOpen() )
+    {
+      // Write sentence into log file
+      nmeaLogFile->write(sentenceIn.toAscii().data());
+    }
+
+  if( sentenceIn.isEmpty() )
+    {
+      return;
+    }
 
   if ( QObject::signalsBlocked() )
     {
@@ -291,7 +347,12 @@ void GpsNmea::slot_sentence(const QString& sentenceIn)
 
   if( ! gpsHash.contains(slst[0]) )
     {
-      qDebug() << "GpsNmea::slot_sentence: No Id found for" << slst[0];
+      if( ! reportedUnknownKeys.contains(slst[0]) )
+        {
+          qWarning() << "GpsNmea::slot_sentence: No Id found for" << slst[0];
+          reportedUnknownKeys.insert(slst[0]);
+        }
+
       return;
     }
 
@@ -409,7 +470,7 @@ void GpsNmea::slot_sentence(const QString& sentenceIn)
 //aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa
 #endif
 
-  // Call the decode methods for the know sentences
+  // Call the decode methods for the known sentences
   switch( gpsHash.value( slst[0] ) )
   {
     case 0: // GPRMC
@@ -481,10 +542,9 @@ void GpsNmea::slot_sentence(const QString& sentenceIn)
 
     default:
 
-      qDebug() << "Unknown GPS sentence:" << sentenceIn;
+      qWarning() << "Unknown GPS sentence:" << sentenceIn;
       return;
   }
-
 }
 
 /**
@@ -523,6 +583,8 @@ void GpsNmea::__ExtractGprmc( const QStringList& slst )
       return;
     }
 
+  _gprmcSeen = true;
+
   //qDebug("%s",slst[2].toLatin1().data());
   if( slst[2] == "A" )
     { /* Data status A=OK, V=warning */
@@ -661,8 +723,15 @@ void GpsNmea::__ExtractGpgga( const QStringList& slst )
     }
 
   if ( slst[6] != "0" && ! slst[6].isEmpty() )
-    { /*a value of 0 means invalid fix and we don't need that one */
-      fixOK( "GGA" );
+    {
+      /*a value of 0 means invalid fix and we don't need that one */
+      if( _gprmcSeen == false )
+        {
+          // Report fix info only, if GPRMC was not received. I saw
+          // sometimes different reportings. GGA said OK, RMC said NOK.
+          fixOK( "GGA" );
+        }
+
       __ExtractTime(slst[1]);
       __ExtractCoord(slst[2],slst[3],slst[4],slst[5]);
       __ExtractAltitude(slst[9],slst[10]);
@@ -670,7 +739,12 @@ void GpsNmea::__ExtractGpgga( const QStringList& slst )
     }
   else if( slst[6] == "0" )
     {
-      fixNOK( "GGA" );
+      if( _gprmcSeen == false )
+        {
+          // Report fix info only, if GPRMC was not received. I saw
+          // sometimes different reportings. GGA said OK, RMC said NOK.
+          fixNOK( "GGA" );
+        }
     }
 }
 
@@ -1486,13 +1560,18 @@ QString GpsNmea::__ExtractConstellation(const QStringList& sentence)
         {
           _lastSatInfo.fixValidity = fix;
 
-          if( fix == 1 )
+          if( _gprmcSeen == false )
             {
-              fixNOK( "GSA" );
-            }
-          else
-            {
-              fixOK( "GSA" );
+              // Report fix info only, if GPRMC was not received. I saw
+              // sometimes different reportings. GSA said OK, RMC said NOK.
+              if( fix == 1 )
+                {
+                  fixNOK( "GSA" );
+                }
+              else
+                {
+                  fixOK( "GSA" );
+                }
             }
         }
     }
@@ -1945,6 +2024,8 @@ void GpsNmea::slot_reset()
   _userAltitudeCorrection = conf->getGpsUserAltitudeCorrection();
   _reportAltitude = true;
 
+#ifndef ANDROID
+
   QString oldDevice = gpsDevice;
 
   if ( gpsDevice != conf->getGpsDevice() )
@@ -1984,6 +2065,9 @@ void GpsNmea::slot_reset()
       // bool soft = conf->getGpsSoftStart();
       // sendLastFix (hard, soft);
     }
+
+#endif
+
 }
 
 #if 0
@@ -2071,12 +2155,17 @@ void GpsNmea::forceReset()
 {
   qDebug("GpsNmea::forceReset()");
 
+#ifndef ANDROID
+
   if ( serial )
     {
       serial->stopGpsReceiving();
       serial->startGpsReceiving();
       slot_reset();
     }
+
+#endif
+
 }
 
 
@@ -2342,3 +2431,162 @@ void GpsNmea::__ExtractSatsInView(const QString& id, const QString& elev,
   sivInfoInternal.append( sivi );
   //qDebug("new sivi info (snr: %d", sivi->db);
 }
+
+/**
+ * called to open the NMEA log file.
+ */
+void GpsNmea::slot_openNmeaLogFile()
+{
+  QString fname = GeneralConfig::instance()->getUserDataDirectory() + "/CumulusNmea.log";
+
+  if( nmeaLogFile == 0 )
+    {
+      if( QFileInfo(fname).exists() )
+        {
+          QFile::remove( fname + ".old" );
+          QFile::rename ( fname, fname + ".old" );
+        }
+
+      nmeaLogFile = new QFile(fname);
+    }
+
+  if( !nmeaLogFile->isOpen() )
+    {
+      bool ok = nmeaLogFile->open( QIODevice::WriteOnly | QIODevice::Text );
+
+      if( ! ok )
+        {
+          qWarning() << "Cannot open file" << fname;
+        }
+    }
+}
+
+/**
+ * called to close the NMEA log file.
+ */
+void GpsNmea::slot_closeNmeaLogFile()
+{
+  if( nmeaLogFile )
+    {
+      if( nmeaLogFile->isOpen() )
+        {
+          nmeaLogFile->close();
+        }
+
+      delete nmeaLogFile;
+      nmeaLogFile = 0;
+    }
+}
+
+#ifdef ANDROID
+
+/**
+ * Handler for custom QEvents posted by the native JNI functions in
+ * jnisupport.cpp. GPS data handling duplicated from other NMEA functions.
+ */
+bool GpsNmea::event(QEvent *event)
+  {
+    if( ! _enableGpsDataProcessing )
+      {
+        return true;
+      }
+
+    // Handles NMEA sentences forwarded by the Android system
+    if( event->type() == QEvent::User + 2 )
+      {
+        GpsNmeaEvent *gpsNmeaEvent = static_cast<GpsNmeaEvent *>(event);
+        slot_sentence( gpsNmeaEvent->sentence() );
+        emit newSentence( gpsNmeaEvent->sentence() );
+        return true;
+      }
+
+    if( event->type() == QEvent::User )
+      {
+        GpsFixEvent *gpsFixEvent = static_cast<GpsFixEvent *>(event);
+
+        // Handle altitude
+        Altitude newAlt(0);
+        double alt = gpsFixEvent->altitude();
+
+        // Consider user's altitude correction
+        newAlt.setMeters( alt + _userAltitudeCorrection.getMeters() );
+
+        _lastGNSSAltitude = newAlt;
+
+        if ( _lastMslAltitude != newAlt )
+          {
+            _lastMslAltitude = newAlt;
+            calcStdAltitude( newAlt );
+            emit newAltitude( _lastMslAltitude, _lastStdAltitude, _lastGNSSAltitude );
+          }
+
+        // Handle position
+        int lat = (int) rint(gpsFixEvent->latitude()  * 600000.0);
+        int lon = (int) rint(gpsFixEvent->longitude() * 600000.0);
+
+        QPoint newPoint (lat, lon);
+
+        if ( _lastCoord != newPoint )
+          {
+            _lastCoord = newPoint;
+            emit newPosition( _lastCoord );
+          }
+
+        // Handle speed
+        Speed newSpeedFix;
+        newSpeedFix.setMps( (double) gpsFixEvent->speed() );
+
+        if ( newSpeedFix != _lastSpeed && fabs((newSpeedFix - _lastSpeed).getMps()) > 0.3 )
+          {
+            // report speed change only if the difference is greater than 0.3m/s, 1.08Km/h
+            _lastSpeed = newSpeedFix;
+            emit newSpeed( _lastSpeed );
+          }
+
+        // Handle heading
+        double newHeadingFix = gpsFixEvent->heading();
+
+        if ( newHeadingFix != _lastHeading )
+          {
+            _lastHeading = newHeadingFix;
+            emit newHeading( _lastHeading );
+          }
+
+        // Handle time
+        QDateTime fix_utc;
+        fix_utc.setMSecsSinceEpoch( gpsFixEvent->time() );
+
+        if( fix_utc.time() != _lastRmcTime )
+          {
+            _lastRmcTime = fix_utc.time();
+            emit newFix( _lastRmcTime );
+          }
+
+        // Handle accuracy
+        return true;
+      }
+
+    // Handles reported status changes of the Android GPS receiver
+    if( event->type() == QEvent::User + 1 )
+      {
+        GpsStatusEvent *gpsStatusEvent = static_cast<GpsStatusEvent *>(event);
+
+        GpsStatus status = static_cast<GpsNmea::GpsStatus>(gpsStatusEvent->status());
+
+        if( status == notConnected )
+          {
+            _slotGpsConnectionOff();
+          }
+        else
+          {
+            _slotGpsConnectionOn();
+          }
+
+        return true;
+      }
+
+  // call the default event processing
+  return QObject::event(event);
+}
+
+#endif /* ANDROID */
