@@ -7,12 +7,12 @@
 ************************************************************************
 **
 **   Copyright (c): 2003 by André Somers
-**                  2008-2011 Axel Pauli
+**                  2008-2012 Axel Pauli
 **
 **   This file is distributed under the terms of the General Public
 **   License. See the file COPYING for more information.
 **
-**   $Id: gpsstatusdialog.h 4626 2011-05-13 20:42:23Z axel $
+**   $Id: gpsstatusdialog.h 4871 2012-01-23 15:30:58Z axel $
 **
 ***********************************************************************/
 
@@ -27,7 +27,9 @@
   * satellites tracked, elevation, azimuth, signal strengths and the NMEA
   * data stream. NMEA data stream can be save into a file on user request.
   *
-  * \date 2003-2011
+  * \date 2003-2012
+  *
+  * \version $Id: gpsstatusdialog.h 4871 2012-01-23 15:30:58Z axel $
   */
 
 #ifndef GPS_STATUS_DIALOG_H
@@ -119,6 +121,18 @@ private slots:
    */
   void slot_updateGpsMessageDisplay();
 
+  /**
+   * Called if close button is pressed.
+   */
+  void slot_Close();
+
+signals:
+
+  /**
+   * This signal is emitted, when the dialog is closed
+   */
+  void closingWidget();
+
 protected:
 
   GpsElevationAzimuthDisplay *elevAziDisplay;
