@@ -3,13 +3,13 @@
                              -------------------
     begin                : Sat Jul 20 2002
     copyright            : (C) 2002      by André Somers
-                               2008-2010 by Axel Pauli
+                               2008-2012 by Axel Pauli
 
     email                : axel@kflog.org
 
     This file is part of Cumulus
 
-    $Id: igclogger.cpp 4089 2010-06-04 16:59:03Z axel $
+    $Id: igclogger.cpp 4979 2012-03-07 13:19:28Z axel $
 
  ***************************************************************************/
 
@@ -22,11 +22,9 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <stdlib.h>
+#include <cstdlib>
 
-#include <QtGlobal>
-#include <QMessageBox>
-#include <QDir>
+#include <QtGui>
 
 #include "igclogger.h"
 #include "gpsnmea.h"
@@ -34,6 +32,7 @@
 
 #include "target.h"
 #include "generalconfig.h"
+#include "mainwindow.h"
 #include "mapcontents.h"
 #include "flighttask.h"
 #include "taskpoint.h"
@@ -43,8 +42,10 @@ IgcLogger* IgcLogger::_theInstance = static_cast<IgcLogger *> (0);
 
 IgcLogger::IgcLogger(QObject* parent) :
   QObject(parent),
+  closeTimer(0),
   _kRecordLogging(false),
-  _backtrack( LimitedList<QStringList> (15) )
+  _backtrack( LimitedList<QStringList>(30) ),
+  _flightMode( Calculator::unknown)
 {
   if ( GeneralConfig::instance()->getLoggerAutostartMode() )
     {
@@ -66,8 +67,11 @@ IgcLogger::IgcLogger(QObject* parent) :
   lastLoggedKRecord = new QTime();
 
   resetTimer = new QTimer( this );
-
   connect( resetTimer, SIGNAL(timeout()), this, SLOT(slotResetLoggingTime()) );
+
+  closeTimer = new QTimer(this);
+  closeTimer->setSingleShot( true );
+  connect( closeTimer, SIGNAL(timeout()), this, SLOT(slotCloseLogFile()) );
 }
 
 IgcLogger::~IgcLogger()
@@ -103,7 +107,6 @@ IgcLogger* IgcLogger::instance()
 void IgcLogger::slotReadConfig()
 {
   // qDebug( "IgcLogger::slotReadConfig()" );
-
   if( _logMode != on )
     {
       // Don't change mode, if logger is switch on
@@ -138,7 +141,6 @@ void IgcLogger::slotResetLoggingTime()
 void IgcLogger::slotMakeFixEntry()
 {
   // qDebug("IgcLogger::slotMakeFixEntry()");
-
   if ( _logMode == off || calculator->samplelist.count() == 0 )
     {
       // make sure logger is not off and and entries are in the sample list
@@ -163,7 +165,10 @@ void IgcLogger::slotMakeFixEntry()
                    QString("%1").arg(GpsNmea::gps->getLastSatInfo().fixAccuracy, 3, 10, QChar('0')) +
                    QString("%1").arg(GpsNmea::gps->getLastSatInfo().satsInUse, 2, 10, QChar('0')) );
 
-  if ( _logMode == standby && calculator->moving() == false )
+  if ( _logMode == standby &&
+       ( calculator->moving() == false ||
+         _flightMode == Calculator::unknown ||
+         _flightMode == Calculator::standstill ) )
     {
       // save B and F record and time in backtrack, if we are not in move
       QString fRecord = "F" +
@@ -190,6 +195,9 @@ void IgcLogger::slotMakeFixEntry()
           // before start with normal logging. Otherwise the first B record is missing.
           _logMode = on;
 
+          // set start date and time of logging
+          startLogging = QDateTime::currentDateTime();
+
           // If log mode was before in standby we have to write out the backtrack entries.
           if( _backtrack.size() > 0 )
             {
@@ -412,6 +420,9 @@ void IgcLogger::CloseFile()
     {
       _logfile.close();
     }
+
+  // reset logger start time
+  startLogging = QDateTime();
 }
 
 /** This function writes the header of the IGC file into the logfile. */
@@ -457,9 +468,10 @@ void IgcLogger::writeHeader()
   _stream << "HFGTYGLIDERTYPE: " << gliderType << "\r\n";
   _stream << "HFGIDGLIDERID: " << gliderRegistration << "\r\n";
   _stream << "HFDTM100GPSDATUM: WSG-1984\r\n";
-  _stream << "HFRFWFIRMWAREVERION: " << CU_VERSION << "\r\n";
+  _stream << "HFRFWFIRMWAREVERION: " << QCoreApplication::applicationVersion() << "\r\n";
   _stream << "HFRHWHARDWAREVERSION: " << HwInfo::instance()->getTypeString() << "\r\n" ;
-  _stream << "HFFTYFRTYPE: Cumulus Version: " << CU_VERSION << ", Qt/X11 Version: " << qVersion() << "\r\n";
+  _stream << "HFFTYFRTYPE: Cumulus Version: " << QCoreApplication::applicationVersion()
+          << ", Qt/X11 Version: " << qVersion() << "\r\n";
   _stream << "HFGPS: UNKNOWN\r\n";
   _stream << "HFPRSPRESSALTSENSOR: UNKNOWN\r\n";
   _stream << "HSCIDCOMPETITIONID: " << gliderCallSign << "\r\n";
@@ -538,13 +550,24 @@ void IgcLogger::slotToggleLogging()
   // qDebug("toggle logging!");
   if ( _logMode == on )
     {
-      int answer = QMessageBox::question( QApplication::activeWindow(),
-                                          tr("Stop Logging?"),
-                                          tr("<html>Are you sure you want<br>stop logging?</html>"),
-                                          QMessageBox::No|QMessageBox::Yes,
-                                          QMessageBox::No );
+      QMessageBox mb( QMessageBox::Question,
+                      tr( "Stop Logging?" ),
+                      tr("<html>Are you sure you want<br>stop logging?</html>"),
+                      QMessageBox::Yes | QMessageBox::No,
+                      MainWindow::mainWindow() );
 
-      if (answer == QMessageBox::Yes)
+      mb.setDefaultButton( QMessageBox::No );
+
+#ifdef ANDROID
+
+      mb.show();
+      QPoint pos = MainWindow::mainWindow()->mapToGlobal( QPoint( MainWindow::mainWindow()->width()/2 - mb.width()/2,
+                                                                  MainWindow::mainWindow()->height()/2 - mb.height()/2 ) );
+      mb.move( pos );
+
+#endif
+
+      if( mb.exec() == QMessageBox::Yes )
         {
           // qDebug("Stopping logging...");
           Stop();
@@ -557,11 +580,24 @@ void IgcLogger::slotToggleLogging()
 
       if( ! calculator->glider() )
         {
-          answer = QMessageBox::warning( QApplication::activeWindow(),
-                     tr("Start Logging?"),
-                     tr("<html>You should select a glider<br>before start logging.<br>Continue start logging?</html>"),
-                     QMessageBox::No|QMessageBox::Yes,
-                     QMessageBox::No );
+          QMessageBox mb( QMessageBox::Warning,
+                          tr( "Start Logging?" ),
+                          tr("<html>You should select a glider<br>before start logging.<br>Continue start logging?</html>"),
+                          QMessageBox::Yes | QMessageBox::No,
+                          MainWindow::mainWindow() );
+
+          mb.setDefaultButton( QMessageBox::No );
+
+    #ifdef ANDROID
+
+          mb.show();
+          QPoint pos = MainWindow::mainWindow()->mapToGlobal( QPoint( MainWindow::mainWindow()->width()/2 - mb.width()/2,
+                                                                      MainWindow::mainWindow()->height()/2 - mb.height()/2 ) );
+          mb.move( pos );
+
+    #endif
+
+          answer = mb.exec();
         }
 
       if( answer == QMessageBox::Yes )
@@ -585,13 +621,24 @@ void IgcLogger::slotNewTaskSelected()
       return;
     }
 
-  int answer = QMessageBox::question( QApplication::activeWindow(),
-                                      tr("Restart Logging?"),
-                                      tr("<html>A new flight task was selected.<br>Restart logging?</html>"),
-                                      QMessageBox::No|QMessageBox::Yes,
-                                      QMessageBox::No );
+  QMessageBox mb( QMessageBox::Warning,
+                  tr( "Restart Logging?" ),
+                  tr("<html>A new flight task was selected.<br>Restart logging?</html>"),
+                  QMessageBox::Yes | QMessageBox::No,
+                  MainWindow::mainWindow() );
 
-  if( answer == QMessageBox::Yes )
+  mb.setDefaultButton( QMessageBox::No );
+
+#ifdef ANDROID
+
+  mb.show();
+  QPoint pos = MainWindow::mainWindow()->mapToGlobal( QPoint( MainWindow::mainWindow()->width()/2 - mb.width()/2,
+                                                              MainWindow::mainWindow()->height()/2 - mb.height()/2 ) );
+  mb.move( pos );
+
+#endif
+
+  if( mb.exec() == QMessageBox::Yes )
     {
       // qDebug("Restarting logging...");
       Stop();
@@ -730,21 +777,50 @@ QString IgcLogger::createFileName(const QString& path)
 {
   int year=QDate::currentDate().year();
   year=year % 10;
-  int month=QDate::currentDate().month();
-  int day=QDate::currentDate().day();
-  QString name=QString::number(year,10) + QString::number(month,13) + QString::number(day,32);
-  name+="X000"; // @AP: one X has to be added for unknown manufacture
+  int month = QDate::currentDate().month();
+  int day = QDate::currentDate().day();
+  QString name = QString::number(year, 10) + QString::number(month, 13) + QString::number(day, 32);
+  name += "X000"; // @AP: one X has to be added for unknown manufacture
 
   int i=1;
-  QString result=name + QString::number(i,36);
+  QString result=name + QString::number(i, 36);
 
-  while (QFile(path + "/" + result.toUpper() + ".IGC").exists())
+  while ( QFile(path + "/" + result.toUpper() + ".IGC").exists() )
     {
       i++;
-      result=name + QString::number(i,36);
+      result = name + QString::number(i, 36);
     }
 
-  flightNumber=i; //store the resulting number so we can use it in the logfile itself
+  flightNumber = i; //store the resulting number so we can use it in the logfile itself
 
   return path + "/" + result.toUpper() + ".IGC";
 }
+
+void IgcLogger::slotFlightModeChanged( Calculator::FlightMode newFlightMode )
+{
+  _flightMode = newFlightMode;
+
+  if( GeneralConfig::instance()->getLoggerAutostartMode() == false )
+    {
+      // Logger auto start mode not active.
+      return;
+    }
+
+  if( newFlightMode == Calculator::standstill || newFlightMode == Calculator::unknown )
+    {
+      // Close logfile after 90s still stand or unknown mode.
+      closeTimer->start(90000);
+    }
+  else
+    {
+      closeTimer->stop();
+    }
+}
+
+void IgcLogger::slotCloseLogFile()
+{
+  if( GeneralConfig::instance()->getLoggerAutostartMode() )
+    {
+      Standby();
+    }
+}
