@@ -8,12 +8,12 @@
  **
  **   Copyright (c):  1999, 2000 by Heiner Lamprecht, Florian Ehinger
  **                   2008 modified by Josua Dietze
- **                   2008-2011 modified by Axel Pauli
+ **                   2008-2012 modified by Axel Pauli
  **
  **   This file is distributed under the terms of the General Public
  **   License. See the file COPYING for more information.
  **
- **   $Id: map.cpp 4741 2011-12-23 14:28:14Z axel $
+ **   $Id: map.cpp 5031 2012-03-23 22:08:50Z axel $
  **
  ***********************************************************************/
 
@@ -30,6 +30,7 @@
 #include "distance.h"
 #include "generalconfig.h"
 #include "gpsnmea.h"
+#include "layout.h"
 #include "hwinfo.h"
 #include "map.h"
 #include "mapcalc.h"
@@ -61,7 +62,7 @@ Map *Map::instance = static_cast<Map *>(0);
 
 Map::Map(QWidget* parent) : QWidget(parent)
 {
-//  qDebug( "Map creation window size is %dx%d, width=%d, height=%d",
+//  qDebug( "Map::Map parent window size is %dx%d, width=%d, height=%d",
 //          size().width(),
 //          size().height(),
 //          size().width(),
@@ -260,12 +261,8 @@ void Map::__displayDetailedItemInfo(const QPoint& current)
   // scale uses unit meter/pixel
   double cs = _globalMapMatrix->getScale(MapMatrix::CurrentScale);
 
-  // snap distance are 15 pixel
-  delta = 15;
-
-#ifdef MAEMO
-  delta = 25;
-#endif
+  // snap distance
+  delta = SnapRadius;
 
   // Manhattan distance is used to found the point.
   int lastDist = 2 * delta + 1;
@@ -548,12 +545,14 @@ void Map::mousePressEvent(QMouseEvent* event)
 
 void Map::mouseMoveEvent( QMouseEvent* event )
 {
+  // qDebug() << "Map::mouseMoveEvent(): Pos=" << event->pos();
+
   if( _mouseMoveIsActive == false &&
      ( GpsNmea::gps->getGpsStatus() != GpsNmea::validFix || calculator->isManualInFlight() ) )
     {
       QPoint dist = _beginMapMove - event->pos();
 
-      if( dist.manhattanLength() > 10 )
+      if( dist.manhattanLength() > 20 )
         {
           // On the N810/N900 we get a lot of move events also on a single
           // mouse press and release action. If we do not filter that,
@@ -646,6 +645,7 @@ void Map::paintEvent(QPaintEvent* event)
 
   // We copy always the content from the m_pixPaintBuffer to the paint device.
   QPainter p(this);
+
   p.drawPixmap( event->rect().left(), event->rect().top(), m_pixPaintBuffer,
                 0, 0, event->rect().width(), event->rect().height() );
 
@@ -973,8 +973,11 @@ void Map::resizeEvent(QResizeEvent* event)
 {
   Q_UNUSED( event )
 
+  // qDebug() << "Map::resizeEvent: NS=" << event->size() << "OS=" << event->oldSize();
+
   // set resize flag
   _isResizeEvent = true;
+
   // start redrawing of map
   slotDraw();
 }
@@ -985,7 +988,8 @@ void Map::__redrawMap(mapLayer fromLayer, bool queueRequest)
 
   static QSize lastSize; // Save the last used window size
 
-  // qDebug("Map::__redrawMap from layer=%d", fromLayer);
+//  qDebug( "Map::__redrawMap from layer=%d, first=%d, isVisible=%d, isEnable=%d, mutex=%d",
+//          fromLayer, first, isVisible(), _isEnable, mutex());
 
   // First call after creation of object can pass
   if( ! isVisible() && ! first )
@@ -1019,7 +1023,6 @@ void Map::__redrawMap(mapLayer fromLayer, bool queueRequest)
     }
 
   // set mutex to block recursive entries and unwanted data modifications.
-
   setMutex(true);
 
   // Check, if a resize event is queued. In this case it must be
@@ -1089,23 +1092,27 @@ void Map::__redrawMap(mapLayer fromLayer, bool queueRequest)
   // unlock mutex
   setMutex(false);
 
-  // Repaints the widget directly by calling paintEvent() immediately,
-  // unless updates are disabled or the widget is hidden.
-  // @AP: Qt 4.3 complains about it, when paintEvent is called directly
+#ifdef QSCROLLER1
+  // Overshoot animation was stopped, reset it
+  _globalMapView->resetScrolling();
+#endif
 
   //QDateTime dt = QDateTime::currentDateTime();
   //QString dtStr = dt.toString(Qt::ISODate);
   // qDebug("%s: Map::__redrawMap: repaint(%dx%d) is called",
   //       dtStr.toAscii().data(), this->rect().width(),this->rect().height() );
 
-  if( ! first )
+  if( first )
     {
-      // suppress the first call otherwise splash screen will disappear
-      repaint( this->rect() );
+      // suppress the first repaint call otherwise splash screen will disappear
+      first = false;
+
+      // Inform MainWindow about first drawing
+      emit firstDrawingFinished();
     }
   else
     {
-      first = false;
+      repaint( m_pixPaintBuffer.rect() );
     }
 
   // @AP: check, if a pending redraw request is active. In this case
@@ -1148,18 +1155,9 @@ void Map::__drawBaseLayer()
   // are no terrain map data available, this is the default map ground color.
   m_pixBaseMap.fill( GeneralConfig::instance()->getTerrainColor(0) );
 
-  // Maps could be loaded before. We do empty the receiver GPS buffer.
-  GpsNmea::gps->readDataFromGps();
-
   // make sure we have all the map files we need loaded
   _globalMapContents->proofeSection();
 
-  // To avoid an overflow in the GPS receiver buffer, we do empty the receiver
-  // buffer after a long action.
-  GpsNmea::gps->readDataFromGps();
-
-  // qDebug("Map::__drawBaseLayer(): zoomFactor=%f", zoomFactor );
-
   double cs = _globalMapMatrix->getScale(MapMatrix::CurrentScale);
 
   // create a pixmap painter
@@ -1170,10 +1168,6 @@ void Map::__drawBaseLayer()
   // first, draw the iso lines
   _globalMapContents->drawIsoList(&baseMapP);
 
-  // To avoid an overflow in the GPS receiver buffer, we do empty the receiver
-  // buffer after a long action.
-  GpsNmea::gps->readDataFromGps();
-
   // next, draw the topographical elements and the cities
   _globalMapContents->drawList(&baseMapP, MapContents::TopoList, drawnElements);
   _globalMapContents->drawList(&baseMapP, MapContents::CityList, m_drawnCityList);
@@ -1579,6 +1573,10 @@ void Map::__drawLabel( QPainter* painter,
                        const bool isLandable ) // is landable?
 {
   // qDebug("LabelName=%s, xShift=%d", name.toLatin1().data(), xShift );
+  if( _globalMapMatrix->getScale(MapMatrix::CurrentScale) >= 120.0 )
+    {
+      return;
+    }
 
   // save the current painter, must be restored before return!!!
   painter->save();
@@ -1587,12 +1585,14 @@ void Map::__drawLabel( QPainter* painter,
   // the labels are good to see at the map.
   QFont font = painter->font();
 
-#ifdef MAEMO
-  font.setPointSize( 24 );
+#if defined ANDROID || defined MAEMO
+  const int fs = 20;
 #else
-  font.setPointSize( 20 );
+  const int fs = 20;
 #endif
 
+  ( font.pointSize() != -1 ) ? font.setPointSize( fs ) : font.setPixelSize( fs );
+
   QString labelText = name;
   Altitude alt = ReachableList::getArrivalAltitude( origP );
 
@@ -1719,11 +1719,15 @@ void Map::_drawCityLabels( QPixmap& pixmap )
 
   QPainter painter(&pixmap);
   QFont font = painter.font();
-#ifdef MAEMO
-  font.setPointSize( 10 );
+
+#if defined ANDROID || defined MAEMO
+  const int fs = 10;
 #else
-  font.setPointSize( 6 );
+  const int fs = 6;
 #endif
+
+  ( font.pointSize() != -1 ) ? font.setPointSize( fs ) : font.setPixelSize( fs );
+
   painter.setFont( font );
   painter.setBrush(Qt::NoBrush);
   painter.setPen(QPen(Qt::black, 2, Qt::SolidLine));
@@ -1867,15 +1871,19 @@ void Map::__drawScale(QPainter& scaleP)
   pen.setWidth(3);
   pen.setCapStyle(Qt::RoundCap);
   scaleP.setPen(pen);
-  QFont f = scaleP.font();
+  QFont font = scaleP.font();
 
-#ifndef MAEMO
-  f.setPointSize(12);
+#ifdef ANDROID
+  const int fs = 12;
+#elif MAEMO
+  const int fs = 14;
 #else
-  f.setPointSize(14);
+  const int fs = 12;
 #endif
 
-  scaleP.setFont(f);
+  ( font.pointSize() != -1 ) ? font.setPointSize( fs ) : font.setPixelSize( fs );
+
+  scaleP.setFont(font);
 
   double scale = _globalMapMatrix->getScale(MapMatrix::CurrentScale);
   Distance barLen;
@@ -2166,12 +2174,12 @@ void Map::__drawOtherAircraft()
 void Map::__drawMostRelevantObject( const Flarm::FlarmStatus& status )
 {
 
-#ifndef MAEMO
-  const int diameter = 22;
-  const int fontPointSize = 18;
-#else
+#if defined ANDROID || defined MAEMO
   const int diameter = 30;
-  const int fontPointSize = 24;
+  const int fontSize = 24;
+#else
+  const int diameter = 22;
+  const int fontSize = 18;
 #endif
 
   if( status.RelativeBearing.isEmpty() ||
@@ -2241,7 +2249,8 @@ void Map::__drawMostRelevantObject( const Flarm::FlarmStatus& status )
   // Set font size used for text painting a little bit bigger, that
   // the labels are good to see at the map.
   QFont font = painter.font();
-  font.setPointSize( fontPointSize );
+
+  ( font.pointSize() != -1 ) ? font.setPointSize( fontSize ) : font.setPixelSize( fontSize );
 
   QString text = Distance::getText( relDistance, false, -1 ) + "/";
 
@@ -2287,14 +2296,14 @@ void Map::__drawMostRelevantObject( const Flarm::FlarmStatus& status )
 void Map::__drawSelectedFlarmObject( const Flarm::FlarmAcft& flarmAcft )
 {
 
-#ifndef MAEMO
-  const int diameter = 22;
-  const int fontPointSize = 18;
-  const int triangle = 26;
-#else
+#if defined ANDROID || defined MAEMO
   const int diameter = 30;
   const int fontPointSize = 24;
   const int triangle = 34;
+#else
+  const int diameter = 22;
+  const int fontPointSize = 18;
+  const int triangle = 26;
 #endif
 
   QPoint other;
@@ -2359,7 +2368,8 @@ void Map::__drawSelectedFlarmObject( const Flarm::FlarmAcft& flarmAcft )
   // Set font size used for text painting a little bit bigger, that
   // the labels are good to see at the map.
   QFont font = painter.font();
-  font.setPointSize( fontPointSize );
+
+  ( font.pointSize() != -1 ) ? font.setPointSize( fontPointSize ) : font.setPixelSize( fontPointSize );
 
   QString text = Distance::getText( distance, false, -1 );
 
@@ -2745,18 +2755,20 @@ void Map::__drawRelBearingInfo()
 
       int pointSize = 16;
 
-    #ifdef MAEMO
+    #if defined ANDROID || defined MAEMO
       pointSize += 4;
     #endif
 
-      font.setPointSize(pointSize);
+      ( font.pointSize() != -1 ) ? font.setPointSize( pointSize ) : font.setPixelSize( pointSize );
+
       font.setBold(true);
 
       QString text = "";
 
       if( relBearing == 0 )
         {
-          text = QString("0") + QChar(Qt::Key_degree);
+          //text = QString("0") + QChar(Qt::Key_degree);
+          text = QString("A");
         }
       else if( relBearing > 0 )
         {
