@@ -5,11 +5,11 @@
 
     copyright            : (C) 2002      by Andre Somers
                                2008      by Josua Dietze
-                               2008-2011 by Axel Pauli
+                               2008-2012 by Axel Pauli
 
     email                : axel@kflog.org
 
-    $Id: mapview.cpp 4741 2011-12-23 14:28:14Z axel $
+    $Id: mapview.cpp 5030 2012-03-23 22:01:55Z axel $
 
  ***************************************************************************/
 
@@ -47,12 +47,15 @@
 MapView::MapView(QWidget *parent) : QWidget(parent)
 {
   setObjectName("MapView");
-
-  resize(parent->size());
-  setContentsMargins(-9,-9,-9,-9);
+  setContentsMargins(-9, -9, -9, -9);
 
   _mainWindow = (MainWindow *)parent;
 
+  if( parent )
+    {
+      resize( parent->size() );
+    }
+
   // Later on the Pretext can change depending on Mode
   GeneralConfig *conf = GeneralConfig::instance();
   _altimeterMode = conf->getAltimeterMode();
@@ -103,7 +106,7 @@ MapView::MapView(QWidget *parent) : QWidget(parent)
   GRLayout->setSpacing(2);
 
   //add Glide Path widget
-  _glidepath = new MapInfoBox( this, conf->getMapFrameColor().name(), true, false, 42 );
+  _glidepath = new MapInfoBox( this, conf->getMapFrameColor().name(), true, false, 40 );
   _glidepathBGColor = wayBar->palette().color(QPalette::Window);
   _glidepath->setValue("-");
   _glidepath->setPreText("Arr");
@@ -262,14 +265,14 @@ MapView::MapView(QWidget *parent) : QWidget(parent)
   _speed2fly = new MapInfoBox( this, "#a6a6a6", true );
   _speed2fly->setPreText("S2f");
   _speed2fly->setValue(tr("Menu"));
-  _speed2fly->setPreUnit( "MU" );
+  _speed2fly->setPreUnit( "|=|" );
   _speed2fly->setVisible( false );
   MSLayout->addWidget( _speed2fly );
   connect(_speed2fly, SIGNAL(mousePress()), this, SLOT(slot_toggleMenu()));
 
    //add menu toggle widget
-  _menuToggle = new MapInfoBox( this, QColor(Qt::gray).name(), false, false, 32 );
-  _menuToggle->setPreTextVisible( false );
+  _menuToggle = new MapInfoBox( this, QColor(Qt::gray).name(), false, false, 30 );
+  _menuToggle->setPreTextWidgetVisible( false );
   _menuToggle->setTextLabelBGColor( "lightGray" );
   _menuToggle->setValue(tr("Menu"));
   MSLayout->addWidget( _menuToggle );
@@ -278,12 +281,43 @@ MapView::MapView(QWidget *parent) : QWidget(parent)
   sideLayout->addWidget( mcBar, 1 );
 
   //--------------------------------------------------------------------
-  //layout for the map
+#ifdef QSCROLLER1
+  // scroll area for the map, used by Android
+  mapArea = new QScrollArea(this);
+  mapArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+  mapArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+  mapArea->setSizePolicy( QSizePolicy::Ignored, QSizePolicy::Ignored );
+  mapArea->setFrameStyle( QFrame::NoFrame );
+
+  centerLayout->addWidget(mapArea, 10);
+
+  _theMap = new Map( mapArea );
+  mapArea->setWidget( _theMap );
+  mapScroller = QScroller::scroller(mapArea);
+
+  QScrollerProperties sp = mapScroller->scrollerProperties();
+  sp.setScrollMetric(QScrollerProperties::OvershootDragDistanceFactor, 0.5);
+  sp.setScrollMetric(QScrollerProperties::MaximumVelocity, QPointF(0.0, 0.0));
+  sp.setScrollMetric(QScrollerProperties::HorizontalOvershootPolicy, QScrollerProperties::OvershootAlwaysOn);
+  sp.setScrollMetric(QScrollerProperties::VerticalOvershootPolicy, QScrollerProperties::OvershootAlwaysOn);
+  mapScroller->setScrollerProperties(sp);
+
+  connect( mapScroller, SIGNAL(stateChanged(QScroller::State)),
+           this, SLOT( slot_scrollerStateChanged(QScroller::State) ) );
+
+  QScroller::grabGesture(mapArea, QScroller::LeftMouseButtonGesture);
+
+#else
+
+  // normal layout for the map
   QBoxLayout *MapLayout = new QHBoxLayout;
   centerLayout->addLayout(MapLayout);
   centerLayout->setStretchFactor( MapLayout, 1 );
   _theMap = new Map(this);
   MapLayout->addWidget(_theMap, 10);
+
+#endif
+
   _theMap->setMode(Map::headUp);
 
 #ifdef FLARM
@@ -301,22 +335,29 @@ MapView::MapView(QWidget *parent) : QWidget(parent)
   _statusbar->setObjectName("status");
   _statusbar->setSizeGripEnabled(false);
 
-#ifndef MAEMO
-  _statusbar->setFixedHeight(20);
-#else
-  _statusbar->setFixedHeight(25);
-#endif
-
   QFont font = _statusbar->font();
   font.setBold(true);
 
-#ifndef MAEMO
-  font.setPixelSize(13);
-#else
-  font.setPixelSize(17);
+  int size = 10;
+
+#ifdef MAEMO
+  size = 16;
 #endif
+
+  if( font.pointSize() != -1 )
+    {
+      font.setPointSize(size);
+    }
+  else
+    {
+      font.setPixelSize(size+4);
+    }
+
   _statusbar->setFont(font);
 
+  QFontMetrics fm(font);
+  _statusbar->setFixedHeight(fm.boundingRect("W°").height() + 5 );
+
   _statusGps = new CuLabel(tr("Man"),_statusbar);
   _statusGps->setLineWidth(0);
   _statusGps->setAlignment(Qt::AlignCenter);
@@ -766,6 +807,11 @@ void MapView::slot_GPSStatus(GpsNmea::GpsStatus status)
   if( status != GpsNmea::notConnected )
     {
       _statusGps->setText( tr( "GPS" ) );
+
+#ifdef QSCROLLER1
+      QScroller::ungrabGesture(mapArea);
+      mapArea->ensureVisible( 0, 0 );
+#endif
     }
   else
     {
@@ -773,6 +819,10 @@ void MapView::slot_GPSStatus(GpsNmea::GpsStatus status)
       _heading->setValue( "-" );
       _speed->setValue( "-" );
       _altitude->setValue( "-" );
+
+#ifdef QSCROLLER1
+      QScroller::grabGesture( mapArea, QScroller::LeftMouseButtonGesture );
+#endif
     }
 
   // Set the color background according to the current GPS status.
@@ -1184,7 +1234,25 @@ void MapView::slot_AltimeterDialog()
   connect( calculator, SIGNAL( newUserAltitude( const Altitude& ) ),
            amDlg, SLOT( slotAltitudeChanged( const Altitude& ) ) );
 
+  connect( amDlg, SIGNAL( closingWidget() ),
+           MainWindow::mainWindow(), SLOT( slotSubWidgetClosed() ) );
+
   amDlg->setVisible(true);
+
+#ifdef ANDROID
+
+  QSize ms = amDlg->minimumSizeHint();
+
+  ms += QSize(10, 10);
+
+  // A dialog is not centered over the parent and not limited in
+  // its size under Android. Therefore this must be done by our self.
+  amDlg->setGeometry( (width() - ms.width()) / 2, (height() - ms.height()) / 2,
+                       ms.width(), ms.height() );
+
+#endif
+
+  MainWindow::setRootWindow( false );
 }
 
 /** Called, if altimeter mode has been changed. */
@@ -1225,7 +1293,25 @@ void MapView::slot_VarioDialog()
   connect( vmDlg, SIGNAL( newTEKAdjust( int ) ),
            calculator->getVario(), SLOT( slotNewTEKAdjust( int ) ) );
 
+  connect( vmDlg, SIGNAL( closingWidget() ),
+           MainWindow::mainWindow(), SLOT( slotSubWidgetClosed() ) );
+
   vmDlg->setVisible(true);
+
+#ifdef ANDROID
+
+  QSize ms = vmDlg->minimumSizeHint();
+
+  ms += QSize(10, 10);
+
+  // A dialog is not centered over the parent and not limited in
+  // its size under Android. Therefore this must be done by our self.
+  vmDlg->setGeometry( (width() - ms.width()) / 2, (height() - ms.height()) / 2,
+                       ms.width(), ms.height() );
+
+#endif
+
+  MainWindow::setRootWindow( false );
 }
 
 /** Opens the GPS status dialog */
@@ -1239,7 +1325,12 @@ void MapView::slot_gpsStatusDialog()
 
   GpsStatusDialog *gpsDlg = new GpsStatusDialog( this );
 
+  connect( gpsDlg, SIGNAL( closingWidget() ),
+           MainWindow::mainWindow(), SLOT( slotSubWidgetClosed() ) );
+
   gpsDlg->setVisible(true);
+
+  MainWindow::setRootWindow( false );
 }
 
 /** Opens the inflight glider settings dialog. */
@@ -1262,7 +1353,25 @@ void MapView::slot_gliderFlightDialog()
   connect( gfDlg, SIGNAL(newWaterAndBugs(const int, const int)),
            calculator, SLOT(slot_WaterAndBugs(const int, const int)) );
 
+  connect( gfDlg, SIGNAL( closingWidget() ),
+           MainWindow::mainWindow(), SLOT( slotSubWidgetClosed() ) );
+
   gfDlg->setVisible(true);
+
+#ifdef ANDROID
+
+  QSize ms = gfDlg->minimumSizeHint();
+
+  ms += QSize(10, 10);
+
+  // A dialog is not centered over the parent and not limited in
+  // its size under Android. Therefore this must be done by our self.
+  gfDlg->setGeometry( (width() - ms.width()) / 2, (height() - ms.height()) / 2,
+                       ms.width(), ms.height() );
+
+#endif
+
+  MainWindow::setRootWindow( false );
 }
 
 /**
@@ -1273,3 +1382,37 @@ void MapView::message( const QString& message, int ms )
 {
   _statusbar->showMessage( message, ms );
 }
+
+#ifdef QSCROLLER1
+
+void MapView::slot_scrollerStateChanged(QScroller::State new_s)
+{
+  // qDebug() << "MapView::slot_scrollerStateChanged: State=" << new_s;
+
+  if( new_s != QScroller::Scrolling )
+    {
+      return;
+    }
+
+  extern MapMatrix* _globalMapMatrix;
+
+  QPointF statePoint = mapScroller->overshootPosition();
+
+  // qDebug() << "mapScroller: calling pause(), StatePoint=" << statePoint;
+  mapScroller->pause();
+
+  QPoint delta = QPoint( (int)statePoint.x(), (int)statePoint.y() );
+  QPoint mapCenter = QPoint( _theMap->width()/2, _theMap->height()/2);
+  mapCenter += delta;
+  QPoint newWgs = _globalMapMatrix->mapToWgs(mapCenter);
+  newWgs = QPoint(newWgs.y(),newWgs.x());
+  calculator->slot_Position( newWgs );
+}
+
+/** Reset the QScroller after pausing and map redraw */
+void MapView::resetScrolling()
+{
+  mapScroller->stop();
+}
+
+#endif
