@@ -7,12 +7,12 @@
 ************************************************************************
 **
 **   Copyright (c):  2002      by Heiner Lamprecht
-**                   2009-2010 by Axel Pauli
+**                   2009-2012 by Axel Pauli
 **
 **   This file is distributed under the terms of the General Public
 **   License. See the file COPYING for more information.
 **
-**   $Id: preflighttasklist.cpp 4589 2011-04-05 18:18:08Z axel $
+**   $Id: preflighttasklist.cpp 4972 2012-03-06 14:24:20Z axel $
 **
 ***********************************************************************/
 
@@ -33,9 +33,11 @@
 
 PreFlightTaskList::PreFlightTaskList( QWidget* parent ) :
   QWidget( parent ),
-  editTask(0)
+  editTask(0),
+  lastFocusWidget(0)
 {
   setObjectName("TaskList");
+
   QVBoxLayout* taskLayout = new QVBoxLayout( this );
   taskLayout->setSpacing(5);
   taskLayout->setMargin(5);
@@ -48,9 +50,12 @@ PreFlightTaskList::PreFlightTaskList( QWidget* parent ) :
   editrow->addWidget(label);
 
   tas = new QSpinBox( this );
+#ifndef ANDROID
   tas->setToolTip( tr("True Air Speed") );
-  tas->setButtonSymbols(QSpinBox::PlusMinus);
-  tas->setRange( 0, 1000);
+#endif
+  tas->setButtonSymbols(QSpinBox::NoButtons);
+  tas->setFocusPolicy(Qt::StrongFocus);
+  tas->setRange( 0, 900);
   tas->setSingleStep( 5 );
   tas->setValue( GeneralConfig::instance()->getTas() );
   tas->setSuffix( Speed::getHorizontalUnitText() );
@@ -60,9 +65,12 @@ PreFlightTaskList::PreFlightTaskList( QWidget* parent ) :
   editrow->addWidget(label);
 
   windDirection = new QSpinBox( this );
+#ifndef ANDROID
   windDirection->setToolTip( tr("Wind Direction") );
-  windDirection->setButtonSymbols(QSpinBox::PlusMinus);
-  windDirection->setRange( 0, 359 );
+#endif
+  windDirection->setButtonSymbols(QSpinBox::NoButtons);
+  windDirection->setFocusPolicy(Qt::StrongFocus);
+  windDirection->setRange( 0, 360 );
   windDirection->setWrapping(true);
   windDirection->setSingleStep( 10 );
   windDirection->setValue( GeneralConfig::instance()->getWindDirection() );
@@ -73,9 +81,12 @@ PreFlightTaskList::PreFlightTaskList( QWidget* parent ) :
   editrow->addWidget(label);
 
   windSpeed = new QSpinBox( this );
+#ifndef ANDROID
   windSpeed->setToolTip( tr("Wind Speed") );
-  windSpeed->setButtonSymbols(QSpinBox::PlusMinus);
-  windSpeed->setRange( 0, 1000 );
+#endif
+  windSpeed->setButtonSymbols(QSpinBox::NoButtons);
+  windSpeed->setFocusPolicy(Qt::StrongFocus);
+  windSpeed->setRange( 0, 900 );
   windSpeed->setValue( GeneralConfig::instance()->getWindSpeed() );
   windSpeed->setSuffix( Speed::getWindUnitText() );
 
@@ -86,35 +97,73 @@ PreFlightTaskList::PreFlightTaskList( QWidget* parent ) :
 
   editrow->addWidget(windSpeed);
 
+  // take a bold font for the plus and minus sign
+  QFont bFont = font();
+  bFont.setBold(true);
+
+  plus  = new QPushButton("+");
+  minus = new QPushButton("-");
+
+#ifndef ANDROID
+  plus->setToolTip( tr("Increase number value") );
+  minus->setToolTip( tr("Decrease number value") );
+#endif
+
+  plus->setFont(bFont);
+  minus->setFont(bFont);
+
+  // The buttons have no focus policy to avoid a focus change during click of them.
+  plus->setFocusPolicy(Qt::NoFocus);
+  minus->setFocusPolicy(Qt::NoFocus);
+
+  editrow->addSpacing(10);
+  editrow->addWidget(plus);
+  editrow->addSpacing(10);
+  editrow->addWidget(minus);
   editrow->addStretch(10);
 
-  QPushButton * cmdNew = new QPushButton(this);
+  QPushButton * cmdNew = new QPushButton;
   cmdNew->setIcon( QIcon(GeneralConfig::instance()->loadPixmap("add.png")) );
   cmdNew->setIconSize(QSize(IconSize, IconSize));
+#ifndef ANDROID
   cmdNew->setToolTip(tr("Define a new task"));
+#endif
   editrow->addWidget(cmdNew);
 
   editrow->addSpacing(10);
-  QPushButton * cmdEdit = new QPushButton(this);
+  QPushButton * cmdEdit = new QPushButton;
   cmdEdit->setIcon( QIcon(GeneralConfig::instance()->loadPixmap("edit_new.png")) );
   cmdEdit->setIconSize(QSize(IconSize, IconSize));
+#ifndef ANDROID
   cmdEdit->setToolTip(tr("Edit selected task"));
+#endif
   editrow->addWidget(cmdEdit);
 
   editrow->addSpacing(10);
-  QPushButton * cmdDel = new QPushButton(this);
+  QPushButton * cmdDel = new QPushButton;
   cmdDel->setIcon( QIcon(GeneralConfig::instance()->loadPixmap("delete.png")) );
   cmdDel->setIconSize(QSize(IconSize, IconSize));
+#ifndef ANDROID
   cmdDel->setToolTip(tr("Remove selected task"));
+#endif
   editrow->addWidget(cmdDel);
 
+  int size = cmdDel->sizeHint().height();
+
+  plus->setMinimumSize(size, size);
+  plus->setMaximumSize(size, size);
+  minus->setMinimumSize(size, size);
+  minus->setMaximumSize(size, size);
+
   splitter = new QSplitter( Qt::Vertical, this );
   splitter->setOpaqueResize( true );
   splitter->setHandleWidth(10);
 
   taskListWidget = new QTreeWidget( splitter );
 
+#ifndef ANDROID
   taskListWidget->setToolTip( tr("Select a flight task") );
+#endif
   taskListWidget->setRootIsDecorated(false);
   taskListWidget->setItemsExpandable(false);
   taskListWidget->setUniformRowHeights(true);
@@ -123,6 +172,10 @@ PreFlightTaskList::PreFlightTaskList( QWidget* parent ) :
   taskListWidget->setSelectionMode(QAbstractItemView::SingleSelection);
   taskListWidget->setColumnCount(4);
 
+#ifdef QSCROLLER
+  QScroller::grabGesture(taskListWidget, QScroller::LeftMouseButtonGesture);
+#endif
+
   QStringList sl;
   sl << tr("No.")
      << tr("Name")
@@ -134,14 +187,25 @@ PreFlightTaskList::PreFlightTaskList( QWidget* parent ) :
 
   taskContent = new TaskListView( splitter, false );
 
+#ifndef ANDROID
+  taskContent->setToolTip( tr("Task display") );
+#endif
+
   taskLayout->addWidget( splitter );
 
   connect(cmdNew, SIGNAL(clicked()), this, SLOT(slotNewTask()));
   connect(cmdEdit, SIGNAL(clicked()), this, SLOT(slotEditTask()));
   connect(cmdDel, SIGNAL(clicked()), this, SLOT(slotDeleteTask()));
-  connect(tas, SIGNAL(valueChanged(int)), this, SLOT(slotTasChanged(int)));
-  connect(windDirection, SIGNAL(valueChanged(int)), this, SLOT(slotWDChanged(int)));
-  connect(windSpeed, SIGNAL(valueChanged(int)), this, SLOT(slotWSChanged(int)));
+  connect(plus, SIGNAL(pressed()), this, SLOT(slotIncrementBox()));
+  connect(minus, SIGNAL(pressed()), this, SLOT(slotDecrementBox()));
+
+  /**
+   * If the plus or minus button is clicked, the focus is changed to the main
+   * window. I don't know why. Therefore the previous focused widget must be
+   * saved, to have an indication, if a spinbox entry should be modified.
+   */
+  connect( QCoreApplication::instance(), SIGNAL(focusChanged( QWidget*, QWidget*)),
+           this, SLOT( slotFocusChanged( QWidget*, QWidget*)) );
 
   connect( taskListWidget, SIGNAL( itemSelectionChanged() ),
            this, SLOT( slotTaskDetails() ) );
@@ -178,17 +242,6 @@ void PreFlightTaskList::showEvent(QShowEvent *)
           // set the splitter line to a new place
           splitter->setSizes(sizeList);
         }
-
-      // Notice user how to add a new task
-      QTreeWidgetItem* selected = taskListWidget->selectedItems().at(0);
-
-      if ( selected->text( 0 ) == " " &&
-           selected->text( 1 ) == tr("(No tasks defined)") )
-        {
-          QMessageBox::information( this,
-                                    tr("Create New Task"),
-                                    tr("Push <b>Plus</b> button to add a task") );
-        }
     }
 
   taskListWidget->resizeColumnToContents(0);
@@ -196,31 +249,66 @@ void PreFlightTaskList::showEvent(QShowEvent *)
   taskListWidget->resizeColumnToContents(2);
 }
 
-/** new value set in TAS spin box */
-void PreFlightTaskList::slotTasChanged( int /* value */ )
+void PreFlightTaskList::slotIncrementBox()
 {
-  // The user has changed the value in the TAS spin box. We
-  // have to initiate an update of the task details.
-  slotTaskDetails();
-  return;
+  if( ! plus->isDown() )
+    {
+      return;
+    }
+
+  // Look which spin box has the focus. Note, focus can be changed by clicking
+  // the connected button. Therefore take old focus widget under account and
+  // set the focus back to the spinbox.
+  QAbstractSpinBox* spinBoxList[3] = {tas, windDirection, windSpeed};
+
+  for( uint i = 0; i < (sizeof(spinBoxList) / sizeof(spinBoxList[0])); i++ )
+    {
+      if( QApplication::focusWidget() == spinBoxList[i] || lastFocusWidget == spinBoxList[i] )
+        {
+          spinBoxList[i]->stepUp();
+          spinBoxList[i]->setFocus();
+          slotTaskDetails();
+
+          // Start repetition timer, to check, if button is longer pressed.
+           QTimer::singleShot(250, this, SLOT(slotIncrementBox()));
+          return;
+        }
+    }
 }
 
-/** value in wind direction spin box has been changed, do update of task list. */
-void PreFlightTaskList::slotWDChanged( int /* value */ )
+void PreFlightTaskList::slotDecrementBox()
 {
-  // The user has changed the value in the wind direction spin box. We
-  // have to initiate an update of the task details.
-  slotTaskDetails();
-  return;
+  if( ! minus->isDown() )
+    {
+      return;
+    }
+
+  // Look which spin box has the focus. Note, focus can be changed by clicking
+  // the connected button. Therefore take old focus widget under account and
+  // set the focus back to the spinbox.
+  QAbstractSpinBox* spinBoxList[3] = {tas, windDirection, windSpeed};
+
+  for( uint i = 0; i < (sizeof(spinBoxList) / sizeof(spinBoxList[0])); i++ )
+    {
+      if( QApplication::focusWidget() == spinBoxList[i] || lastFocusWidget == spinBoxList[i] )
+        {
+          spinBoxList[i]->stepDown();
+          spinBoxList[i]->setFocus();
+          slotTaskDetails();
+
+          // Start repetition timer, to check, if button is longer pressed.
+          QTimer::singleShot(250, this, SLOT(slotDecrementBox()));
+          return;
+        }
+    }
 }
 
-/** value in wind speed spin box has been changed, do update of task list. */
-void PreFlightTaskList::slotWSChanged( int /* value */ )
+void PreFlightTaskList::slotFocusChanged( QWidget* oldWidget, QWidget* newWidget)
 {
-  // The user has changed the value in the wind speed spin box. We
-  // have to initiate an update of the task details.
-  slotTaskDetails();
-  return;
+  Q_UNUSED(newWidget)
+
+  // We save the old widget, which has just lost the focus.
+  lastFocusWidget = oldWidget;
 }
 
 void PreFlightTaskList::slotTaskDetails()
@@ -248,7 +336,7 @@ void PreFlightTaskList::slotTaskDetails()
   task->setSpeed( tas->value() );
 
   // update wind parameters, can be changed in the meantime by the user
-  task->setWindDirection( windDirection->value() );
+  task->setWindDirection( windDirection->value() % 360 );
   task->setWindSpeed( windSpeed->value() );
 
   taskContent->slot_setTask( task );
@@ -311,7 +399,7 @@ bool PreFlightTaskList::loadTaskList()
 
   taskNames.clear();
 
-#warning task list file 'tasks.tsk' is stored at User Data Directory
+#warning "task list file 'tasks.tsk' is stored at User Data Directory"
 
   // currently hard coded file name
   QFile f( GeneralConfig::instance()->getUserDataDirectory() + "/tasks.tsk" );
@@ -561,11 +649,24 @@ void PreFlightTaskList::slotDeleteTask()
       return;
     }
 
-  int answer= QMessageBox::question(this,tr("Delete Task"),
-                                    tr("Delete the selected task?"),
-                                    QMessageBox::No, QMessageBox::Yes );
+  QMessageBox mb( QMessageBox::Question,
+                  tr( "Delete Task?" ),
+                  tr( "Delete the selected task?" ),
+                  QMessageBox::Yes | QMessageBox::No,
+                  this );
 
-  if ( answer != QMessageBox::Yes )
+  mb.setDefaultButton( QMessageBox::No );
+
+#ifdef ANDROID
+
+  mb.show();
+  QPoint pos = mapToGlobal(QPoint( width()/2  - mb.width()/2,
+                                   height()/2 - mb.height()/2 ));
+  mb.move( pos );
+
+#endif
+
+  if ( mb.exec() != QMessageBox::Yes )
     {
       return;
     }
