@@ -6,12 +6,12 @@
 **
 ************************************************************************
 **
-**   Copyright (c):  2009-2010 by Axel Pauli
+**   Copyright (c):  2009-2012 by Axel Pauli
 **
 **   This file is distributed under the terms of the General Public
 **   License. See the file COPYING for more information.
 **
-**   $Id: settingspageterraincolors.cpp 4718 2011-12-08 19:24:23Z axel $
+**   $Id: settingspageterraincolors.cpp 5010 2012-03-16 12:44:07Z axel $
 **
 ***********************************************************************/
 
@@ -21,10 +21,11 @@
 #include "generalconfig.h"
 #include "settingspageterraincolors.h"
 #include "elevationcolorimage.h"
-
+#include "varspinbox.h"
 
 SettingsPageTerrainColors::SettingsPageTerrainColors(QWidget *parent) :
-  QWidget(parent)
+  QWidget(parent),
+  m_autoSip( true )
 {
   setObjectName("SettingsPageTerrainColors");
 
@@ -88,7 +89,7 @@ SettingsPageTerrainColors::SettingsPageTerrainColors(QWidget *parent) :
   };
 
   // determine pixmap size to be used for icons in dependency of the used font
-  int size = font().pointSize() - 2;
+  int size = font().pointSize();
   pixmapSize = QSize( size, size );
   QPixmap pixmap(pixmapSize);
 
@@ -217,12 +218,12 @@ SettingsPageTerrainColors::SettingsPageTerrainColors(QWidget *parent) :
   offsetLayout->addStretch( 10 );
 
   QVBoxLayout *spinboxLayout = new QVBoxLayout;
+
   elevationOffset = new QSpinBox;
-  elevationOffset->setButtonSymbols(QAbstractSpinBox::PlusMinus);
   elevationOffset->setSingleStep(1);
   elevationOffset->setRange(-50, 50);
-
-  spinboxLayout->addWidget(elevationOffset);
+  VarSpinBox* hspin = new VarSpinBox( elevationOffset );
+  spinboxLayout->addWidget(hspin);
   setOffsetBox->setLayout(spinboxLayout);
 
   topLayout->addLayout( offsetLayout );
@@ -234,6 +235,18 @@ SettingsPageTerrainColors::~SettingsPageTerrainColors()
 {
 }
 
+void SettingsPageTerrainColors::showEvent(QShowEvent *)
+{
+  // Switch off automatic software input panel popup
+  m_autoSip = qApp->autoSipEnabled();
+  qApp->setAutoSipEnabled( false );
+}
+
+void SettingsPageTerrainColors::hideEvent( QHideEvent *)
+{
+  qApp->setAutoSipEnabled( m_autoSip );
+}
+
 /**
  * Called to edit the color of the combo box selection
  */
